<?php
/**
 * Created by WHMCSModule Networks (https://www.whmcsmodule.net/).
 *
 * PHP version 7.x and above
 *
 * @author WHMCSModule Networks <info@whmcsmodule.net>
 *
 * @author Zare (Hydra Communications Ltd) <team@zare.com>
 *
 * @see https://www.zare.com/
 *
 * @see Dcouemtnation https://zare.com/knowledgebase/article/75/whmcs-reseller-module-guide
 *
 */
if (!defined("WHMCS")) {die("This file cannot be accessed directly");}
use App;
use WHMCS\Database\Capsule;
use WHMCS\View\Menu\Item as MenuItem;
use WHMCS\Module\Server\ZareServer\ZareServer;
use WHMCS\Module\Server\ZareServer\Helper;
use WHMCS\Module\Server\ZareServer\vmManager;
use WHMCS\Module\Server\ZareServer\clientManager;
use WHMCS\Module\Server\ZareServer\adminManager;
use WHMCS\Module\Server\ZareServer\usageUpdate;
use WHMCS\Module\Server\ZareServer\configOptions;

/**
 * Returns metadata for the ZareServer service.
 *
 * This function provides essential information about the ZareServer service,
 * such as its display name, API version, and whether it requires a server.
 *
 * @return array An associative array containing metadata about the ZareServer service:
 *               - 'DisplayName' (string): A human-readable name for the service.
 *               - 'APIVersion' (string): The version of the API used by the service.
 *               - 'RequiresServer' (bool): Indicates whether the service requires a server.
 */
function ZareServer_MetaData()
{
    return [
        'DisplayName' => 'Zare Servers',
        'APIVersion' => '1.0',
        'RequiresServer' => true,
    ];
}

/**
 * Retrieves configuration options for a server based on the given parameters.
 *
 * This function queries the database to find a server that belongs to a specific
 * server group and is not disabled. If a valid server is found, it initializes
 * a `configOptions` object with the server's access hash and returns the configuration options.
 *
 * @param array $params An associative array of parameters used for configuration.
 * @return array|null Returns the configuration options if the server is found and
 *                    valid, or null if no valid server is found.
 */
function ZareServer_ConfigOptions(array $params)
{
    $server = Capsule::table('tblservers')
        ->join('tblservergroupsrel', 'tblservergroupsrel.serverid', '=', 'tblservers.id')
        ->where('tblservergroupsrel.groupid', App::getFromRequest('servergroup'))
        ->where('tblservers.disabled', '0')
        ->first();
        if(isset($server->accesshash)){
          $params['serveraccesshash'] = $server->accesshash;
          $manager = new configOptions($params);
          return $manager->configs();
        }
}

/**
 * Tests the connection to the ZareServer API.
 *
 * This function attempts to connect to the ZareServer using the provided
 * parameters and checks if the connection is successful by retrieving server
 * OS IDs. It returns an associative array indicating the success status
 * and any error message encountered during the process.
 *
 * @param array $params An associative array containing connection parameters
 *                      such as API key and server URL.
 * @return array An associative array with two keys:
 *               - 'success' (bool): Indicates whether the connection was successful.
 *               - 'error' (string): Contains an error message if the connection failed,
 *                 or an empty string if it was successful.
 */
function ZareServer_TestConnection(array $params)
{
  try {
    $manager = new ZareServer($params);
    $account = $manager->getServerOSIds();
    if($account['response_code'] == '200'){
    $success = true;
  } else {
    $errorMsg = 'Invalid API details';
  }
} catch (\Exception $e) {
       $success = false;
       $errorMsg = $e->getMessage();
   }
   return ['success' => $success, 'error' => $errorMsg];
 }

 /**
  * Terminates an account or service based on its status.
  *
  * This function checks the status of the account or service. If the status is
  * either 'Active' or 'Suspended', it attempts to terminate the account using
  * the `vmManager` class. If the status is neither 'Active' nor 'Suspended', it
  * returns an error message indicating that termination is not possible.
  *
  * @param array $params An associative array containing parameters related to the account,
  *                      including its status.
  * @return string A message indicating the result of the termination attempt, either
  *                success or an error message.
  */
  function ZareServer_TerminateAccount(array $params)
  {
      if ($params['status'] != 'Active' && $params['status'] != 'Suspended') {
          return 'Cannot terminate service';
      }
      try{
        $vmManager = new vmManager($params);
        return $vmManager->terminate();
      } catch (\Exception $e) {
          return $e->getMessage();
      }
  }

  /**
   * Suspends an account or service based on its current status.
   *
   * This function checks if the account or service is in a 'Terminated' state.
   * If it is, the function returns an error message indicating that suspension is
   * not possible. Otherwise, it attempts to suspend the service by calling the
   * `stop()` method of the `vmManager` class.
   *
   * @param array $params An associative array containing parameters related to the account,
   *                      including its status.
   * @return string A message indicating the result of the suspension attempt, either
   *                success or an error message.
   */
  function ZareServer_SuspendAccount(array $params)
  {
    if ($params['status'] == 'Terminated') {
        return 'Cannot suspend terminated service';
    }
      try {
        $vmManager = new vmManager($params);
        return $vmManager->stop();
      } catch (\Exception $e) {
          return $e->getMessage();
      }
  }

  /**
   * Unsuspends an account or service based on its current status.
   *
   * This function checks if the account or service is in a 'Terminated' state.
   * If it is, the function returns an error message indicating that unsuspension
   * is not possible. Otherwise, it attempts to unsuspend the service by calling
   * the `start()` method of the `vmManager` class.
   *
   * @param array $params An associative array containing parameters related to the account,
   *                      including its status.
   * @return string A message indicating the result of the unsuspension attempt, either
   *                success or an error message.
   */
  function ZareServer_UnsuspendAccount(array $params)
  {
    if ($params['status'] == 'Terminated') {
        return 'Cannot unsuspend terminated service';
    }
      try {
        $vmManager = new vmManager($params);
        return $vmManager->start();
      } catch (\Exception $e) {
          return $e->getMessage();
      }
  }

  /**
   * Stops a service or virtual machine based on the provided parameters.
   *
   * This function initializes the `vmManager` class with the given parameters
   * and attempts to stop the service or VM by calling the `stop()` method.
   * If an error occurs during the process, it catches the exception and returns
   * the error message.
   *
   * @param array $params An associative array containing parameters needed to
   *                      stop the service or VM.
   * @return string The result of the stop attempt, either a success message
   *                or an error message if an exception is thrown.
   */
  function ZareServer_Stop(array $params)
  {
      try {
        $vmManager = new vmManager($params);
        return $vmManager->stop();
      } catch (\Exception $e) {
          return $e->getMessage();
      }
  }

  /**
   * Starts a service or virtual machine based on the provided parameters.
   *
   * This function initializes the `vmManager` class with the given parameters
   * and attempts to start the service or VM by calling the `start()` method.
   * If an error occurs during the process, it catches the exception and returns
   * the error message.
   *
   * @param array $params An associative array containing parameters needed to
   *                      start the service or VM.
   * @return string The result of the start attempt, either a success message
   *                or an error message if an exception is thrown.
   */
  function ZareServer_Start(array $params)
  {
      try {
        $vmManager = new vmManager($params);
        return $vmManager->start();
      } catch (\Exception $e) {
          return $e->getMessage();
      }
  }

  /**
   * Reboots a service or virtual machine based on the provided parameters.
   *
   * This function initializes the `vmManager` class with the given parameters
   * and attempts to reboot the service or VM by calling the `reboot()` method.
   * If an error occurs during the process, it catches the exception and returns
   * the error message.
   *
   * @param array $params An associative array containing parameters needed to
   *                      reboot the service or VM.
   * @return string The result of the reboot attempt, either a success message
   *                or an error message if an exception is thrown.
   */
  function ZareServer_Reboot(array $params)
  {
      try {
        $vmManager = new vmManager($params);
        return $vmManager->reboot();
      } catch (\Exception $e) {
          return $e->getMessage();
      }
  }

  /**
   * Provides an array of custom buttons for the admin interface.
   *
   * This function returns an associative array where each key represents
   * the label of a custom button and each value represents the action
   * to be performed when the button is clicked.
   *
   * @return array An associative array of custom buttons with their labels and actions.
   */
  function ZareServer_AdminCustomButtonArray()
  {
      return [
          'Start Server' => 'Start',
          'Stop Server' => 'Stop',
          'Reboot Server' => 'Reboot',
      ];
  }

  /**
   * Updates the reverse DNS record for an IP address.
   *
   * This function retrieves the IP address and reverse DNS record from the
   * POST request data, initializes the `vmManager` class with these parameters,
   * and attempts to set the reverse DNS record by calling the `rdns()` method.
   * If an error occurs, it catches the exception and returns the error message.
   *
   * @param array $params An associative array containing additional parameters needed for the operation.
   * @return string The result of the reverse DNS update attempt, either a success message or an error message.
   */
  function ZareServer_ReverseDNS( $params )
  {
  try {
    $params["ipsddr"] = $_POST['ipsddr'];
    $params["rdns"] = $_POST['rdns'];
    $vmManager = new vmManager($params);
    return $vmManager->rdns();

  } catch (\Exception $e) {
            return $e->getMessage();
      }
  }

  /**
   * Configures or manages firewall settings for a service or virtual machine.
   *
   * This function retrieves firewall-related parameters from the POST request,
   * initializes the `vmManager` class with these parameters, and attempts to
   * apply firewall settings by calling the `firewall()` method. If an error
   * occurs, it catches the exception and returns the error message.
   *
   * @param array $params An associative array containing additional parameters needed for the operation.
   * @return string The result of the firewall configuration attempt, either a success message
   *                or an error message if an exception is thrown.
   */
  function ZareServer_Firewall( $params )
  {
  try {
    $params["fipsddr"] = $_POST['fipsddr'];
    $params["state"] = $_POST['state'];
    $params["sensitivity"] = $_POST['sensitivity'];
    $vmManager = new vmManager($params);
    return $vmManager->firewall();

  } catch (\Exception $e) {
            return $e->getMessage();
      }
  }

  /**
   * Reinstalls the operating system or image for a service or virtual machine.
   *
   * This function retrieves the OS image identifier from the POST request,
   * initializes the `vmManager` class with these parameters, and attempts to
   * perform the reinstallation by calling the `rebuild()` method. If an error
   * occurs, it catches the exception and returns the error message.
   *
   * @param array $params An associative array containing additional parameters needed for the operation.
   * @return string The result of the reinstallation attempt, either a success message
   *                or an error message if an exception is thrown.
   */
  function ZareServer_reInstall( $params )
  {
  try {
    $params["osimg"] = $_POST['osimg'];
    $vmManager = new vmManager($params);
    return $vmManager->rebuild();

  } catch (\Exception $e) {
            return $e->getMessage();
      }
  }

  /**
   * Populates admin interface fields for the services tab.
   *
   * This function retrieves the server ID, checks the server status, and if the server
   * is active and the server ID is available, it fetches additional data such as OS images
   * and provides it for rendering in the admin interface. It also handles specific actions
   * like opening a console. If an error occurs, it returns an error message.
   *
   * @param array $params An associative array containing parameters needed for the operation.
   * @return array An associative array containing data for rendering the admin interface or an error message.
   */
  function ZareServer_AdminServicesTabFields(array $params)
  {
    $serverId = $params['model']->serviceProperties->get('serverId|Server Id') ?? "";
      if ($params['status'] != 'Active') {
          return [];
      }
      if (empty($serverId)) {
          return [];
      }
      try {
        $manager = new ZareServer($params);
        $LANG = Helper::getLang();
        $action = $_REQUEST["subaction"];
        if (!empty($action) && ($action == "console")) {
          if (ob_get_length())
          {
            ob_clean();
          }

        $VnConsole = $manager->getConsole($serverId);
        if(empty($VnConsole)){
        } else {
        include 'lib/console.php';
        }
        die();
        }

        $getServerOSIds = $manager->getServerOSIds()['response'];
        $params['image'] = array();
      foreach ($getServerOSIds as $item) {
        if($item['id'] == "1000"){
          $params['image'][$item['id']] = $item['name']. ' ('.$LANG["rescueos"].') ';
        } else {
          $params['image'][$item['id']] = $item['name'];
        }
      }
        $adminManager = new adminManager($params);
          return $adminManager->adminarea();
      } catch (\Exception $e) {
          return ['error' => $e->getMessage()];
      }
  }

  /**
   * Handles saving actions for the admin services tab.
   *
   * This function performs different actions based on the 'act' request parameter,
   * such as reinstallation, reverse DNS, or firewall configuration. It calls the
   * corresponding functions for these actions and handles exceptions if they occur.
   *
   * @param array $params An associative array containing parameters needed for the operation.
   * @return string A success message or an error message if an exception is thrown.
   */
  function ZareServer_AdminServicesTabFieldsSave( $params )
  {
  try {
    if($_REQUEST['act'] == 'reInstall'){
    ZareServer_reInstall( $params );
    }
  if($_REQUEST['act'] == 'rDNS'){
  ZareServer_ReverseDNS( $params );
  }
  if($_REQUEST['act'] == 'Firewall'){
  ZareServer_Firewall( $params );
  }
} catch (\Exception $e) {
          return $e->getMessage();
      }
      return 'success';
  }

  /**
   * Handles client area operations for a specific service.
   *
   * This function processes actions related to the client area, such as fetching
   * console information, managing AJAX actions, and preparing data for rendering
   * client-side templates. It returns data or templates based on the operations performed.
   *
   * @param array $params An associative array containing parameters needed for the operation.
   * @return array An associative array containing template data or an error template if an exception is thrown.
   */
  function ZareServer_ClientArea(array $params)
  {
    $serverId = $params['model']->serviceProperties->get('serverId|Server Id') ?? "";
      if ($params['status'] != 'Active') {
          return [];
      }
      if (empty($serverId)) {
          return [];
      }
          try {
          Helper::clientAreaPrimarySidebarHook($params);
          $manager = new ZareServer($params);

          $LANG = Helper::getLang();
          $action = App::getFromRequest('subaction');
          if (!empty($action) && ($action == "console")) {
            if (ob_get_length())
            {
              ob_clean();
            }

          $VnConsole = $manager->getConsole($serverId);
          if(empty($VnConsole)){
          } else {
          include 'lib/console.php';
          }
          die();
          }
          if (!empty($action)) {
              Helper::ajaxAction($params, $action);
          }
          $getServerOSIds = $manager->getServerOSIds()['response'];
          $params['image'] = array();
        foreach ($getServerOSIds as $item) {
          if($item['id'] == "1000"){
          $params['image'][$item['id']] = $item['name']. ' ('.$LANG["rescueos"].') ';
          } else {
            $params['image'][$item['id']] = $item['name'];
          }
        }
          $clientManager = new clientManager($params);
          $vm = $clientManager->getData('details');
          return [
              'templatefile' => 'templates/overview.tpl',
              'templateVariables' => [
                  'vm' => $vm,
                  '_LANG' => Helper::getLang(),
              ],
          ];
      } catch (\Exception $e) {
          return ['templatefile' => 'templates/error.tpl'];
      }
  }

  /**
   * Updates usage information for a service.
   *
   * This function creates an instance of the `usageUpdate` class and calls its
   * `usage` method to perform the update. If an exception occurs during this process,
   * it catches the exception and returns the error message.
   *
   * @param array $params An associative array containing parameters needed for the usage update.
   * @return string|null Returns the error message if an exception is thrown, otherwise returns null.
   */
  function ZareServer_UsageUpdate(array $params)
  {
      try {
        $manager = new usageUpdate($params);
        $manager->usage();
      } catch (\Exception $e) {
          return $e->getMessage();
      }
  }
