<?php
/**
 * Created by WHMCSModule Networks (https://www.whmcsmodule.net/).
 *
 * PHP version 7.x and above
 *
 * @author WHMCSModule Networks <info@whmcsmodule.net>
 *
 * @author Zare (Hydra Communications Ltd) <team@zare.com>
 *
 * @see https://www.zare.com/
 *
 * @see Dcouemtnation https://zare.com/knowledgebase/article/75/whmcs-reseller-module-guide
 *
 */
namespace WHMCS\Module\Server\ZareServer;
if (!defined("WHMCS")) {die("This file cannot be accessed directly");}

use WHMCS\View\Menu\Item as MenuItem;
use WHMCS\Module\Server\ZareServer\ZareServer;
use WHMCS\Module\Server\ZareServer\ajaxAction;

/**
 * Class Helper
 *
 * Provides utility functions for language management, AJAX actions, and client area sidebar customization.
 */
class Helper
{

  /**
 * Retrieves the language array based on the current session or configuration.
 *
 * This method checks the current language setting from the session or configuration, loads the appropriate
 * language file from the 'lang' directory, and returns the language array.
 *
 * @return array The language array containing translated strings.
 */
  public static function getLang()
  {
      $languageDir = dirname(__FILE__) . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'lang' . DIRECTORY_SEPARATOR;
      $config = isset($GLOBALS['CONFIG']) ? $GLOBALS['CONFIG'] : [];
      $language = isset($_SESSION['Language']) ? $_SESSION['Language'] : (isset($config['Language']) ? $config['Language'] : 'english');
      $languageFile = file_exists($languageDir . $language . '.php') ? $language : 'english';
      if (file_exists($languageDir . 'english.php')) {
          include $languageDir . 'english.php';
      }
      require $languageDir . $languageFile . '.php';
      return isset($_LANG) ? $_LANG : [];
  }

  /**
   * Handles AJAX actions by creating an instance of ajaxAction and invoking the specified action method.
   *
   * This method manages AJAX requests, including language retrieval, error handling, and response formatting.
   *
   * @param array $params Parameters for the AJAX action.
   * @param string $action The name of the method to call on the ajaxAction instance.
   * @return void Outputs JSON-encoded response and terminates the script.
   */
  public static function ajaxAction(array $params, string $action)
  {
      ob_clean();
      try {
          $manager = new ajaxAction($params);
          $_LANG = self::getLang();

          if (method_exists($manager, $action)) {
              $details = $manager->$action();
              if ($details['response']['error']) {
                $results['result'] = 'failure';
                $results['message'] = $details['response']['message'];
              } else {
                $results['message'] = $details['response']['message'] ?? $_LANG['ajax']['action']['success'];
                switch ($action) {
                  case "getIpAddresses":
                  $results = $details;
                  break;
                  case "getAttakLogs":
                  $results = $details;
                  break;
                }
              }

          } else {
              $results['result'] = 'failure';
              $results['message'] = $_LANG['ajax']['action']['not_valid'];
          }

          echo json_encode($results);
          die;
      } catch (\Exception $e) {
          echo json_encode(['result' => 'failure', 'message' => $e->getMessage()]);
          die;
      }
  }

  /**
   * Customizes the primary sidebar in the client area by modifying menu items.
   *
   * This method adds or modifies menu items in the client area primary sidebar to include a link
   * to the product details page based on the provided service ID.
   *
   * @param array $params Parameters including the service ID.
   * @return void
   */
  public static function clientAreaPrimarySidebarHook(array $params)
  {
      add_hook('ClientAreaPrimarySidebar', 1, function (MenuItem $primarySidebar) use ($params) {
          $_LANG = Helper::getLang();
          $panel = $primarySidebar->getChild('Service Details Overview');
          if (is_a($panel, 'WHMCS\View\Menu\Item')) {
              $panel = $panel->getChild('Information');
              if (is_a($panel, 'WHMCS\View\Menu\Item')) {
                  $panel->setUri("clientarea.php?action=productdetails&id={$params['serviceid']}");
                  $panel->setAttributes([]);
              }
          }
      });
  }

}
