<?php
/**
 * Created by WHMCSModule Networks (https://www.whmcsmodule.net/).
 *
 * PHP version 7.x and above
 *
 * @author WHMCSModule Networks <info@whmcsmodule.net>
 *
 * @author Zare (Hydra Communications Ltd) <team@zare.com>
 *
 * @see https://www.zare.com/
 *
 * @see Dcouemtnation https://zare.com/knowledgebase/article/75/whmcs-reseller-module-guide
 *
 */
namespace WHMCS\Module\Server\ZareServer;
if (!defined("WHMCS")) {die("This file cannot be accessed directly");}

use WHMCS\Module\Server\ZareServer\ZareServer;
use WHMCS\Module\Server\ZareServer\Helper;

/**
 * Class adminManager
 *
 * Admin area management for Zare Servers in WHMCS.
 */
class adminManager
{
  /** @var ZareServer $manager API client for interacting with ZareServer. */
  private $manager;

  /** @var array $params Parameters for initializing the class. */
  private $params;

  /** @var string $rescuePassword The rescue password for the server. */
  private $rescuePassword;

  /** @var array $_LANG Language array for translation. */
  private $_LANG;

  /**
   * Constructor
   * Initializes the adminManager class, sets up the ZareServer manager,
   * retrieves the rescue password from service properties, and loads language settings.
   *
   * @param array $params Parameters including model and service properties.
   */
  public function __construct(array $params)
  {
    $this->manager = new ZareServer($params);
    $this->params = $params;
    $this->rescuePassword = $params['model']->serviceProperties->get('rescue|Rescue Password') ?? "";
    $this->_LANG = Helper::getLang();
  }

  /**
   * Admin area management
   * Handles various administrative tasks in the admin area. This method could be extended to handle
   * different admin actions based on the request or parameters.
   *
   * @return array Response data for admin actions.
   */
public function adminarea(){
  global $aInt;
  $instanceId = $this->params['model']->serviceProperties->get('serverId|Server Id') ?? "";
  $details = $this->manager->GetServerData($instanceId)['response'];
if($details['installing'] == "1"){
  $Reinstall = '
  <link rel="stylesheet" href="../modules/servers/ZareServer/templates/assets/css/style.css">
  <div class="error">
    <div class="error__icon">
        <svg fill="none" height="24" viewBox="0 0 24 24" width="24" xmlns="http://www.w3.org/2000/svg"><path d="m13 13h-2v-6h2zm0 4h-2v-2h2zm-1-15c-1.3132 0-2.61358.25866-3.82683.7612-1.21326.50255-2.31565 1.23915-3.24424 2.16773-1.87536 1.87537-2.92893 4.41891-2.92893 7.07107 0 2.6522 1.05357 5.1957 2.92893 7.0711.92859.9286 2.03098 1.6651 3.24424 2.1677 1.21325.5025 2.51363.7612 3.82683.7612 2.6522 0 5.1957-1.0536 7.0711-2.9289 1.8753-1.8754 2.9289-4.4189 2.9289-7.0711 0-1.3132-.2587-2.61358-.7612-3.82683-.5026-1.21326-1.2391-2.31565-2.1677-3.24424-.9286-.92858-2.031-1.66518-3.2443-2.16773-1.2132-.50254-2.5136-.7612-3.8268-.7612z" fill="#393a37"></path></svg>
    </div>
    <div class="error__title">'.$this->_LANG['ServerInstalling'].'</div>
</div>
  ';
  $menu[$this->_LANG["Reinstalling"]] = $Reinstall;
  return $menu;
} elseif(!empty($this->rescuePassword)){
  $rescue = '
  <link rel="stylesheet" href="../modules/servers/ZareServer/templates/assets/css/style.css">
  <div class="error">
    <div class="error__icon">
        <svg fill="none" height="24" viewBox="0 0 24 24" width="24" xmlns="http://www.w3.org/2000/svg"><path d="m13 13h-2v-6h2zm0 4h-2v-2h2zm-1-15c-1.3132 0-2.61358.25866-3.82683.7612-1.21326.50255-2.31565 1.23915-3.24424 2.16773-1.87536 1.87537-2.92893 4.41891-2.92893 7.07107 0 2.6522 1.05357 5.1957 2.92893 7.0711.92859.9286 2.03098 1.6651 3.24424 2.1677 1.21325.5025 2.51363.7612 3.82683.7612 2.6522 0 5.1957-1.0536 7.0711-2.9289 1.8753-1.8754 2.9289-4.4189 2.9289-7.0711 0-1.3132-.2587-2.61358-.7612-3.82683-.5026-1.21326-1.2391-2.31565-2.1677-3.24424-.9286-.92858-2.031-1.66518-3.2443-2.16773-1.2132-.50254-2.5136-.7612-3.8268-.7612z" fill="#393a37"></path></svg>
    </div>
    <div class="error__title">'.$this->_LANG['ServerRescue'].'</div>
</div>
<div style="display: flex; justify-content: space-between;">
<table class="datatable"
style="width:400px; text-align:center;margin-top:20px;border-spacing: 5px;border-collapse: separate;">
<tbody>
<tr> <th>'.$this->_LANG["Hostname"].'</th> <td>'.$details["hostname"].'</td> </tr>
<tr> <th>'.$this->_LANG["VMId"].'</th> <td>'.$details["id"].'</td> </tr>
</tbody></table><table class="datatable"
style="width:400px; text-align:center;margin-top:20px;border-spacing: 5px;border-collapse: separate;">
<tbody>
<tr> <th>'.$this->_LANG["IPAddress"].'</th> <td>'.$details["IP"].'</td> </tr>
<tr> <th>'.$this->_LANG["password"].'</th> <td>'.$this->rescuePassword.'</td> </tr>
</tbody></table>
</div>
  ';
  $menu[$this->_LANG["rescue"]] = $rescue;
  return $menu;
} else {
$drives = json_decode($details['drives'], true);
$formattedDrives = [];
foreach ($drives as $drive) {
    $quantity = $drive['quantity'];
    $size = $drive['drive_size'];
    $type = $drive['drive_type'];
    $formattedDrives[] = "$quantity x $size $type";
}
$DrivesFormat = implode(', ', $formattedDrives);

      if (!empty($details["additional_ips"])) {
        foreach ($details['additional_ips'] as $ip) {
          $state = ($ip["always_on"] == "0") ? $this->_LANG['firewall']['state_redirect_on_attack'] : $this->_LANG['firewall']['state_always_on'];
            $tableData[] = array($ip["ip"], $ip["subnet"], $ip["gateway"], $ip["netmask"], ucfirst($ip["firewall_sensitivity"]), $state, $ip["rdns"]);
        }
        $aInt->sortableTableInit("nopagination");
        $interfaceInfo = $aInt->sortableTable(array($this->_LANG["IPAddress"], $this->_LANG["subnet"], $this->_LANG["gateway"], $this->_LANG["netmask"], $this->_LANG["firewall_sensitivity"], $this->_LANG["firewall"]["state"], $this->_LANG["reversePTR"]), $tableData);
      }

$locations = $this->manager->getLocations();
foreach ($locations as $regions) {
  if($regions['id'] == $details['location']){
    $details['location'] = $regions['long_name'];
    $details['flag'] = explode(", ", $regions['long_name'])[1];
    break;
  }
}


  $output = '
  <div style="display: flex; justify-content: space-between;">
  <table class="datatable"
  style="width:400px; text-align:center;margin-top:20px;border-spacing: 5px;border-collapse: separate;">
  <tbody>
  <tr> <th>'.$this->_LANG["Hostname"].'</th> <td>'.$details["hostname"].'</td> </tr>
  <tr> <th>'.$this->_LANG["VMId"].'</th> <td>'.$details["id"].'</td> </tr>
  <tr> <th>'.$this->_LANG["Template"].'</th> <td><img src="../modules/servers/ZareServer/templates/assets/img/os/'.$this->manager->OSMatch($details['OS']).'.png" style="height: 24px;margin-right:5px;"> '.$details['OS'].'</td> </tr>
  <tr> <th>'.$this->_LANG["IPAddress"].'</th> <td>'.$details["IP"].'</td> </tr>
  <tr> <th>'.$this->_LANG["password"].'</th> <td>'.$details['root_password'].'</td> </tr>
  <tr> <th>'.$this->_LANG["Status"].'</th> <td>'.$details['status'].'</td> </tr>';
  if (!empty($details['ipmi_ip'])) {
  $output .= '<tr> <th>'.$this->_LANG["server"]["console"].'</th>
  <td>
  <button class="btn btn-primary" onclick="openConsole(event)">
          <i class="fas fa-terminal"></i> '.$this->_LANG["server"]["consoleop"].'
      </button>
  </td> </tr>';
  }
  $output .= '
  </tbody></table><table class="datatable"
  style="width:400px; text-align:center;margin-top:20px;border-spacing: 5px;border-collapse: separate;">
  <tbody>
  <tr> <th>'.$this->_LANG["Processor"].'</th> <td>'.$details["processor"].'</td> </tr>
  <tr> <th>'.$this->_LANG["Drives"].'</th> <td>'.$DrivesFormat.'</td> </tr>
  <tr> <th>'.$this->_LANG["memory"].'</th> <td>'.$details["memory"].' '.$this->_LANG["GB"].'</td> </tr>
 <tr> <th>'.$this->_LANG["location"].'</th> <td><img src="../modules/servers/ZareServer/templates/assets/img/flags/'.$details['flag'].'.png" style="height: 18px;margin-right:5px;"> '.$details['location'].'</td> </tr>
  <tr> <th>'.$this->_LANG["TotalTraffic"].'</th> <td>'.$details["bandwidth"].' '.$this->_LANG["TB"].'</td> </tr>
    <tr> <th>'.$this->_LANG["Uplink"].'</th> <td>'.$details["port_speed"].' '.$this->_LANG["Mbps"].'</td> </tr>
  </tbody></table>
</div>
  ';
  $output .="<script>
function openConsole(event) {
    event.preventDefault();
    window.open(window.location.href + '&subaction=console', '', 'width=10,height=10');
}
</script>";

  $Rebuild = '<table width="100%">
  <tr><td colspan="4">
  <select name="osimg" class="form-control input-200 input-inline">
  <option value="">'.$this->_LANG["selOS"].'</option>
  ';
  foreach ($this->params['image'] as $osid => $osname) {
  $Rebuild .= '<option value="'.$osid.'">'.$osname.'</option>';
  }
  $Rebuild .= '
  </select>
  <input name="act" type="submit" value="reInstall" class="btn btn-sm btn-info">
  </td></tr></table>';

  $Bandwidth = '<ul class="nav nav-tabs">
  <li class="active"><a data-toggle="tab" href="#Traffic">'.$this->_LANG["Traffic"].'</a></li>
  <li><a data-toggle="tab" href="#PPS">'.$this->_LANG["PPS"].'</a></li>
  <li><a data-toggle="tab" href="#PU">'.$this->_LANG["PU"].'</a></li>
  <li><a data-toggle="tab" href="#PE">'.$this->_LANG["PE"].'</a></li>
  <li><a data-toggle="tab" href="#APS">'.$this->_LANG["APS"].'</a></li>
  <li><a data-toggle="tab" href="#PD">'.$this->_LANG["PD"].'</a></li>
</ul>

<div class="tab-content">
  <div id="Traffic" class="tab-pane fade in active">
    <p><img src="'.$this->manager->serverBW($instanceId, "port_bits", "month")['response']['base64'].'" style="width:100%"></p>
  </div>
  <div id="PPS" class="tab-pane fade">
  <p><img src="'.$this->manager->serverBW($instanceId, "port_upkts", "month")['response']['base64'].'" style="width:100%"></p>
  </div>
  <div id="PU" class="tab-pane fade">
  <p><img src="'.$this->manager->serverBW($instanceId, "port_percent", "month")['response']['base64'].'" style="width:100%"></p>
  </div>
  <div id="PE" class="tab-pane fade">
  <p><img src="'.$this->manager->serverBW($instanceId, "port_errors", "month")['response']['base64'].'" style="width:100%"></p>
  </div>
  <div id="APS" class="tab-pane fade">
  <p><img src="'.$this->manager->serverBW($instanceId, "port_pktsize", "month")['response']['base64'].'" style="width:100%"></p>
  </div>
  <div id="PD" class="tab-pane fade">
  <p><img src="'.$this->manager->serverBW($instanceId, "port_discards", "month")['response']['base64'].'" style="width:100%"></p>
  </div>
</div>';

  $ReverseDNS = '<table width="100%">
  <tr><td colspan="4">
  <select name="ipsddr" class="form-control input-200 input-inline">
  <option value="">'.$this->_LANG["selIPAddress"].'</option>
  ';
  foreach ($details['additional_ips'] as $IPList) {
  $ReverseDNS .= '<option value="'.$IPList['ip'].'">'.$IPList['ip'].'</option>';
  }
  $ReverseDNS .= '
  </select>
  <input type="text" placeholder="'.$this->_LANG["rdnsval"].'" class="form-control input-200 input-inline" name="rdns">
  <input name="act" type="submit" value="rDNS" class="btn btn-sm btn-info">
  </td></tr></table>';

  $Firewall = '<table width="100%">
  <tr><td colspan="4">
  <select name="fipsddr" class="form-control input-200 input-inline">
  <option value="">'.$this->_LANG["selIPAddress"].'</option>
  ';
  foreach ($details['additional_ips'] as $IPList) {
  $Firewall .= '<option value="'.$IPList['ip'].'">'.$IPList['ip'].'</option>';
  }
  $Firewall .= '
  </select>
  <select name="state" class="form-control input-200 input-inline">
              <option value="">'.$this->_LANG['firewall']['Selectstate'].'</option>
              <option value="always">'.$this->_LANG['firewall']['state_always_on'].'</option>
              <option value="redirect">'.$this->_LANG['firewall']['state_redirect_on_attack'].'</option>
            </select>
            <select name="sensitivity" class="form-control input-200 input-inline">
                          <option value="">'.$this->_LANG['firewall']['Selectsensitivity'].'</option>
                          <option value="low">'.$this->_LANG['firewall']['high'].'</option>
                          <option value="medium">'.$this->_LANG['firewall']['medium'].'</option>
                          <option value="normal">'.$this->_LANG['firewall']['normal'].'</option>
                          <option value="high">'.$this->_LANG['firewall']['low'].'</option>
                        </select>
  <input name="act" type="submit" value="Firewall" class="btn btn-sm btn-danger">
  </td></tr></table>';

  $menu[$this->_LANG["VmInfo"]] = $output;
  $menu[$this->_LANG["Interface"]] = $interfaceInfo;
  $menu[$this->_LANG["Rebuild"]] = $Rebuild;
   $menu[$this->_LANG["reversePTR"]] = $ReverseDNS;
   $menu[$this->_LANG["Firewall"]] = $Firewall;
   $menu[$this->_LANG["Bandwidth"]] = $Bandwidth;
  if($instanceId){
    return $menu;
  }

}

}
}
