<?php
/**
 * Created by WHMCSModule Networks (https://www.whmcsmodule.net/).
 *
 * PHP version 7.x and above
 *
 * @author WHMCSModule Networks <info@whmcsmodule.net>
 *
 * @author Zare (Hydra Communications Ltd) <team@zare.com>
 *
 * @see https://www.zare.com/
 *
 * @see Dcouemtnation https://zare.com/knowledgebase/article/75/whmcs-reseller-module-guide
 *
 */
namespace WHMCS\Module\Server\ZareServer;
if (!defined("WHMCS")) {die("This file cannot be accessed directly");}

use WHMCS\Database\Capsule;
use WHMCS\Module\Server\ZareServer\Helper;
use WHMCS\Module\Server\ZareServer\ZareServer;

/**
 * Class ajaxAction
 *
 * Performed various Ajax actions for Zare Servers in WHMCS.
 */
class ajaxAction
{
  /** @var ZareServer $manager API client for interacting with ZareServer. */
  private $manager;

  /** @var array $params Parameters for initializing the class. */
  private $params;

  /** @var int $instanceId The ID of the server instance. */
  private $instanceId;

  /** @var int $serviceid The service id for the service. */
  private $serviceid;

  /** @var array $_LANG Language array for translation. */
  private $_LANG;

  /**
   * Constructor
   * Initializes the ajaxAction class, sets up the ZareServer manager, retrieves instance ID and service ID,
   * and loads language settings.
   *
   * @param array $params Parameters including model and service ID.
   */
  public function __construct(array $params)
  {
    $this->manager = new ZareServer($params);
    $this->instanceId = $params['model']->serviceProperties->get('serverId|Server Id') ?? "";
    $this->serviceid = $params['serviceid'];
    $this->params = $params;
    $this->_LANG = Helper::getLang();
  }

  /**
   * Start the server
   * Sends a command to the ZareServer API to power on the server.
   *
   * @return array API response.
   */
  public function StartServer()
  {
  return $this->manager->serverAction($this->instanceId, "poweron");
  }

  /**
   * Restart the server
   * Sends a command to the ZareServer API to restart (power cycle) the server.
   * Clears the rescue password if the restart is successful.
   *
   * @return array API response.
   */
  public function RestartServer()
  {
    $serverAction = $this->manager->serverAction($this->instanceId, "powercycle");
    if(isset($serverAction['response']['success'])){
      $this->params['model']->serviceProperties->save(['rescue|Rescue Password' => ""]);
    }
    return $serverAction;
  }

  /**
   * Stop the server
   * Sends a command to the ZareServer API to power off the server.
   *
   * @return array API response.
   */
  public function StopServer()
  {
    return $this->manager->serverAction($this->instanceId, "poweroff");
  }

  /**
   * Rebuild the server
   * Rebuilds the server with a new OS image. Updates the rescue password or service credentials based on the OS type.
   *
   * @return array API response.
   */
  public function RebuildServer()
  {
      $os_ids = filter_input(INPUT_POST, 'osimg', FILTER_SANITIZE_STRING);
      $rebuild = $this->manager->provisionDedicatedServer($this->instanceId, $os_ids );

      if (isset($rebuild['response']['success'])) {
      $getOsImage = $this->manager->getServerOSIds()['response'];
      $username = 'root';
      foreach ($getOsImage as $item) {
          if ($item['id'] == $os_ids) {
              $username = ($item['type'] == 'win') ? 'Administrator' : 'root';
              break;
          }
      }
      if($os_ids == "1000"){
          $this->params['model']->serviceProperties->save(['rescue|Rescue Password' => $rebuild['response']['root_password']]);
      } else {
        $postData = [
            'serviceid' => $this->serviceid,
            'serviceusername' => $username,
            'servicepassword' => $rebuild['response']['root_password'],
        ];
        localAPI('UpdateClientProduct', $postData);
      }
  }
      return $rebuild;
  }

  /**
   * Get IP addresses
   * Retrieves and formats the list of IP addresses associated with the server, including additional details.
   *
   * @return array Formatted IP addresses with additional details for display.
   */
  public function getIpAddresses()
  {
      $details =  $this->manager->GetServerData($this->instanceId)['response']['additional_ips'];
        foreach ($details as $ip) {
          $state = ($ip["always_on"] == "0") ? $this->_LANG['firewall']['state_redirect_on_attack'] : $this->_LANG['firewall']['state_always_on'];
          $sensitivity = ucfirst($ip["firewall_sensitivity"]);
          $title = $this->_LANG["firewall"]["state"].': '.$state.' & '.$this->_LANG["firewall_sensitivity"].': '.$sensitivity;
            $output[] = [
              '<a href="#" data-toggle="tooltip" title="'.$title.'">'.$ip["ip"].'</a>',
              $ip["gateway"],
              $ip["netmask"],
              $ip["rdns"],
              '<button class="btn btn-sm btn-primary editIp" data-toggle="modal" data-target="#editPtrModal" data-ip="'.$ip['ip'].'" data-ptr="'.$ip["rdns"].'"><i class="btn-icon fa fa-pencil fa-sm"></i></button>
              <button class="btn btn-sm btn-danger ddosIp" data-toggle="modal" data-target="#editfirewallModal" data-ip="'.$ip['ip'].'"><i class="btn-icon fas fa-shield-alt fa-sm"></i></button>
              '
            ];
        }
        $ips['data'] = (!empty($output)) ? $output : [];
        return $ips;
  }

  /**
   * Edit IP address PTR record
   * Updates the PTR (reverse DNS) record for a specified IP address.
   *
   * @return array API response.
   */
  public function editIp()
  {
      return $this->manager->ModifyIPaddress($this->instanceId, filter_input(INPUT_POST, 'ip', FILTER_SANITIZE_STRING), filter_input(INPUT_POST, 'ptr', FILTER_SANITIZE_STRING));
  }

  /**
   * Get attack logs
   * Retrieves and formats attack logs from the firewall for display.
   *
   * @return array Formatted attack logs.
   */
  public function getAttakLogs()
  {
      $firewall =  $this->manager->getFirewallHistory($this->instanceId)['response']['firewall'];
      if(isset($firewall)){
        foreach ($firewall as $logs) {
            $output[] = [
              $logs["ip"],
              ($logs["port"] == "0") ? 'Unknown' : $logs["port"],
              nl2br(str_replace('; ', ";\n", $logs["raw"])),
              $logs["size"].'Mbps',
              $logs["pps"],
              date('d-m-Y H:i:s', $logs["timestamp"]),
            ];
        }
      }
        $ips['data'] = (!empty($output)) ? $output : [];
        return $ips;
  }

  /**
   * Edit firewall settings
   * Updates firewall settings based on the type (sensitivity or state) and action specified.
   *
   * @return array API response.
   */
  public function editFirewall()
  {
    $ftype = filter_input(INPUT_POST, 'ftype', FILTER_SANITIZE_STRING);
    $faction = filter_input(INPUT_POST, 'faction', FILTER_SANITIZE_STRING);
    $ip = filter_input(INPUT_POST, 'ip', FILTER_SANITIZE_STRING);
    if($ftype == "sensitivity"){
      return $this->manager->SetFirewallSensitivity($this->instanceId, $ip, $faction);
    }
    if($ftype == "state"){
      return $this->manager->SetFirewallState($this->instanceId, $ip, $faction);
    }
  }


}
