<?php
/**
 * Created by WHMCSModule Networks (https://www.whmcsmodule.net/).
 *
 * PHP version 7.x and above
 *
 * @author WHMCSModule Networks <info@whmcsmodule.net>
 *
 * @author Zare (Hydra Communications Ltd) <team@zare.com>
 *
 * @see https://www.zare.com/
 *
 * @see Dcouemtnation https://zare.com/knowledgebase/article/75/whmcs-reseller-module-guide
 *
 */
namespace WHMCS\Module\Server\ZareServer;
if (!defined("WHMCS")) {die("This file cannot be accessed directly");}
use WHMCS\Module\Server\ZareServer\Helper;
use WHMCS\Module\Server\ZareServer\ZareServer;

/**
 * Class clientManager
 *
 * Manages interactions with ZareServer and retrieves server details.
 * Handles data retrieval for various server details, including hardware, IPs, and usage statistics.
 */
class clientManager
{
  /** @var ZareServer $manager API client for interacting with ZareServer. */
  private $manager;

  /** @var array $params Parameters for initializing the class. */
  private $params;

  /** @var string $instanceId The ID of the server instance. */
  private $instanceId;

  /** @var string $rescuePassword The rescue password for the server. */
  private $rescuePassword;

  /** @var array $_LANG Language array for translation. */
  private $_LANG;

  /**
   * Constructor for clientManager.
   *
   * Initializes the ZareServer client and sets up class properties.
   *
   * @param array $params Parameters including configuration details for ZareServer.
   */
  public function __construct(array $params)
  {
    $this->manager = new ZareServer($params);
    $this->params = $params;
    $this->instanceId = $params['model']->serviceProperties->get('serverId|Server Id') ?? "";
    $this->rescuePassword = $params['model']->serviceProperties->get('rescue|Rescue Password') ?? "";
    $this->_LANG = Helper::getLang();
  }

  /**
   * Retrieves data based on the specified page.
   *
   * Depending on the page parameter, fetches server details and formats them.
   * Returns an array with the server details or an empty array on failure.
   *
   * @param string $page The page for which to retrieve data (e.g., 'details').
   * @return array An array containing the server details or an empty array.
   */
  public function getData($page)
  {
      try {
          switch ($page) {
              case 'details':
                  $details = $this->manager->GetServerData($this->instanceId);
                  if ($details['response_code'] == '200') {
                      $details = $details['response'];
                      if($details['installing'] == "1"){
                        $Reinstall = 'yes';
                      }

                      $drives = json_decode($details['drives'], true);
                      $formattedDrives = [];
                      foreach ($drives as $drive) {
                          $quantity = $drive['quantity'];
                          $size = $drive['drive_size'];
                          $type = $drive['drive_type'];
                          $formattedDrives[] = "$quantity x $size $type";
                      }
                      $DrivesFormat = implode(', ', $formattedDrives);

                      if (!empty($details["additional_ips"])) {
                        foreach ($details['additional_ips'] as $ip) {
                            $tableData[] = ["ip" => $ip["ip"], "rdns" => $ip["rdns"], "subnet" => $ip["subnet"]];
                        }
                        }

                        $locations = $this->manager->getLocations();
                        foreach ($locations as $regions) {
                          if($regions['id'] == $details['location']){
                            $details['location'] = $regions['long_name'];
                            $details['flag'] = explode(", ", $regions['long_name'])[1];
                            break;
                          }
                        }

                        if (!empty($this->rescuePassword)) {
                            $rescue = 'yes';
                            $details['rescue_password'] = $this->rescuePassword;
                        }

                        if (!empty($details['ipmi_ip'])) {
                            $consoleyes = 'yes';
                        }

                      $data['details'] = [
                          'location' => $details['location'],
                          'hostname' => $details['hostname'],
                          'password' => $details['rescue_password'],
                          'osimg' => $this->manager->OSMatch($details['OS']),
                          'flag' => $details['flag'],
                          'rescue' => $rescue ?? "",
                          'console' => $consoleyes ?? "",
                          'ip' => $details["IP"],
                          'ip_rdns' => $tableData,
                          'server_install' => $Reinstall ?? "",
                          'template' => $details['OS'],
                          'diskSize' => $DrivesFormat,
                          'status' => $details['status'],
                          'processor' => $details['processor'],
                          'memory' => $details['memory'].' '.$this->_LANG["GB"],
                          'TotalTraffic' => $details["bandwidth"].' '.$this->_LANG["TB"],
                          'Uplink' => $details["port_speed"].' '.$this->_LANG["Mbps"],
                          'osImage' => $this->params['image'],
                          'port_bits' => $this->manager->serverBW($this->instanceId, "port_bits", "month")['response']['base64'],
                          'port_upkts' => $this->manager->serverBW($this->instanceId, "port_upkts", "month")['response']['base64'],
                          'port_percent' => $this->manager->serverBW($this->instanceId, "port_percent", "month")['response']['base64'],
                          'port_errors' => $this->manager->serverBW($this->instanceId, "port_errors", "month")['response']['base64'],
                          'port_pktsize' => $this->manager->serverBW($this->instanceId, "port_pktsize", "month")['response']['base64'],
                          'port_discards' => $this->manager->serverBW($this->instanceId, "port_discards", "month")['response']['base64'],
                      ];
                  }
                  break;
          }
      } catch (\Exception $e) {
          return [];
      }

      return $data;
  }

}
