<?php
/**
 * Created by WHMCSModule Networks (https://www.whmcsmodule.net/).
 *
 * PHP version 7.x and above
 *
 * @author WHMCSModule Networks <info@whmcsmodule.net>
 *
 * @author Zare (Hydra Communications Ltd) <team@zare.com>
 *
 * @see https://www.zare.com/
 *
 * @see Dcouemtnation https://zare.com/knowledgebase/article/75/whmcs-reseller-module-guide
 *
 */
namespace WHMCS\Module\Server\ZareServer;
if (!defined("WHMCS")) {die("This file cannot be accessed directly");}
use WHMCS\Database\Capsule;
use WHMCS\Module\Server\ZareServer\ZareServer;

/**
 * Class usageUpdate
 *
 * Performed Bandwidth usages for Zare Servers in WHMCS.
 */
class usageUpdate
{

  /** @var ZareServer $manager API client for server operations. */
  private $manager;

  /** @var array $params Parameters passed to the class, including configuration settings. */
  private $params;

  /**
 * Constructor for usageUpdate.
 *
 * Initializes the ZareServer client and sets up class parameters.
 *
 * @param array $params Parameters including server ID and other configuration details.
 */
  public function __construct(array $params)
  {
    $this->manager = new ZareServer($params);
    $this->params = $params;
  }

  /**
   * Updates bandwidth usage for active services associated with the ZareServer.
   *
   * Retrieves active hosting services from WHMCS, fetches server data from ZareServer,
   * and updates the bandwidth usage and limits in the WHMCS database.
   *
   * The method performs the following actions:
   * 1. Queries active hosting services related to ZareServer from the WHMCS database.
   * 2. For each service, retrieves the server instance ID and server data from ZareServer.
   * 3. Formats the bandwidth data and updates the relevant records in WHMCS.
   *
   * This method assumes that the `ZareServer` client has methods for fetching server data and formatting bandwidth.
   *
   * @return void
   */
  public function usage(){
    $services = Capsule::table('tblhosting')
    ->join('tblproducts', 'tblproducts.id', '=', 'tblhosting.packageid')
    ->where('tblhosting.domainstatus', 'Active')
    ->where('tblhosting.server', $this->params['serverid'])
    ->where('tblproducts.servertype', 'ZareServer')->get(['tblhosting.id','tblhosting.packageid']);

  foreach ($services as $whmcsData) {
    $serviceId = $whmcsData->id;
    $packageId = $whmcsData->packageid;

    $tblcustomfields = Capsule::table('tblcustomfields')
    ->where('tblcustomfields.relid', $packageId)
    ->where('tblcustomfields.fieldname', 'serverId|Server Id')->value('id');

    $instanceId = Capsule::table('tblcustomfieldsvalues')
    ->where('tblcustomfieldsvalues.relid', $serviceId)
    ->where('tblcustomfieldsvalues.fieldid', $tblcustomfields)->value('value');

    $details = $this->manager->GetServerData($instanceId)['response'];
    $TotalTraffic = $this->manager->formatSizeTBtoMB($details['bandwidth']);
    $Outgoing = $this->manager->formatSizeGBtoMB($details['total_used_bw']);
    Capsule::table('tblhosting')
            ->where('server', $this->params['serverid'])
            ->where('id', $serviceId)
            ->where('packageid', $packageId)
            ->update([
                'bwusage' => $Outgoing,
                'bwlimit' => $TotalTraffic,
                'lastupdate' => Capsule::raw('now()'),
            ]);
  }
  }

}
