<?php
/**
 * Created by WHMCSModule Networks (https://www.whmcsmodule.net/).
 *
 * PHP version 7.x and above
 *
 * @author WHMCSModule Networks <info@whmcsmodule.net>
 *
 * @author Zare (Hydra Communications Ltd) <team@zare.com>
 *
 * @see https://www.zare.com/
 *
 * @see Dcouemtnation https://zare.com/knowledgebase/article/75/whmcs-reseller-module-guide
 *
 */
namespace WHMCS\Module\Server\ZareServer;
if (!defined("WHMCS")) {die("This file cannot be accessed directly");}
use WHMCS\Database\Capsule;
use WHMCS\Module\Server\ZareServer\ZareServer;
use WHMCS\Module\Server\ZareServer\Helper;


/**
 * Class vmManager
 *
 * Performed various actions such as start, stop etc for Zare Servers in WHMCS.
 */
class vmManager
{
    /** @var ZareServer $manager API client for server operations. */
    private $manager;

    /** @var string $instanceId ID of the server instance. */
    private $instanceId;

    /** @var array $params Parameters passed to the class, including configuration settings. */
    private $params;

    /** @var string $_LANG Language setting retrieved from Helper. */
    private $_LANG;

    /**
     * Constructor for vmManager.
     *
     * Initializes the ZareServer client and sets up instance properties.
     *
     * @param array $params Parameters including the server configuration and details.
     */
    public function __construct(array $params)
    {
      $this->manager = new ZareServer($params);
      $this->_LANG = Helper::getLang();
      $this->params = $params;
      $this->instanceId = $params['model']->serviceProperties->get('serverId|Server Id') ?? "";
    }

    /**
     * Powers off the server.
     *
     * @return string "success" if the action was successful, otherwise an error message.
     */
  public function stop(){
    $serverAction = $this->manager->serverAction($this->instanceId, "poweroff")['response'];
    return isset($serverAction['success']) ? "success" : $serverAction['message'];
  }

  /**
   * Powers on the server.
   *
   * @return string "success" if the action was successful, otherwise an error message.
   */
  public function start(){
    $serverAction =  $this->manager->serverAction($this->instanceId, "poweron")['response'];
    return isset($serverAction['success']) ? "success" : $serverAction['message'];
  }

  /**
   * Reboots the server.
   *
   * Optionally saves the rescue password if the reboot is successful.
   *
   * @return string "success" if the action was successful, otherwise an error message.
   */
  public function reboot(){
    $serverAction =  $this->manager->serverAction($this->instanceId, "powercycle")['response'];
    if(isset($serverAction['success'])){
      $this->params['model']->serviceProperties->save(['rescue|Rescue Password' => ""]);
    }
    return isset($serverAction['success']) ? "success" : $serverAction['message'];
  }

  /**
   * Terminates the server.
   *
   * @return string "success" if the action was successful, otherwise an error message.
   */
  public function terminate(){
    $serverAction =  $this->manager->CancelServer($this->instanceId, "end")['response'];
    return isset($serverAction['success']) ? "success" : $serverAction['message'];
  }

  /**
   * Rebuilds the server with a new operating system.
   *
   * Updates the rescue password or client product information based on the OS image.
   *
   * @return string "success" if the action was successful, otherwise an error message.
   */
  public function rebuild(){
    $os_ids = $this->params['osimg'];
    $provisionDedicatedServer = $this->manager->provisionDedicatedServer($this->instanceId, $os_ids)['response'];
    if (isset($provisionDedicatedServer['success'])) {
    $getOsImage = $this->manager->getServerOSIds()['response'];
    $username = 'root';
    foreach ($getOsImage as $item) {
        if ($item['id'] == $os_ids) {
            $username = ($item['type'] == 'win') ? 'Administrator' : 'root';
            break;
        }
    }
    if($os_ids == "1000"){
        $this->params['model']->serviceProperties->save(['rescue|Rescue Password' => $provisionDedicatedServer['root_password']]);
    } else {
      $postData = [
          'serviceid' => $this->params['serviceid'],
          'serviceusername' => $username,
          'servicepassword' => $provisionDedicatedServer['root_password'],
      ];
      localAPI('UpdateClientProduct', $postData);
    }
}
    return isset($provisionDedicatedServer['success']) ? "success" : $provisionDedicatedServer['message'];
  }

  /**
   * Modifies the reverse DNS (rDNS) record for the server's IP address.
   *
   * @return string "success" if the action was successful, otherwise an error message.
   */
  public function rdns(){
    $ip = $this->params['ipsddr'];
    $DNSValue = $this->params['rdns'];
    $data = $this->manager->ModifyIPaddress($this->instanceId, $ip, $DNSValue);
    return isset($data['success']) ? "success" : $data['message'];
  }

  /**
   * Configures firewall settings for the server.
   *
   * Sets firewall sensitivity and state based on provided parameters.
   *
   * @return string "success" if the action was successful.
   */
  public function firewall(){
    $ip = $this->params['fipsddr'];
    $state = $this->params['state'];
    $sensitivity = $this->params['sensitivity'];
    if(!empty($sensitivity)){
      $this->manager->SetFirewallSensitivity($this->instanceId, $ip, $sensitivity);
    }
    if(!empty($state)){
      $state = ($state == "always") ? 1 : 0;
      $this->manager->SetFirewallState($this->instanceId, $ip, $state);
    }
    return "success";
  }

}
